/*
 * Copyright (c) 2001-2003 Shiman Associates Inc. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include <errno.h>
#include <string.h>
#include <sys/audio.h>
#include "mas/mas_dpi.h"
#include "aixppc_defines.h"

/* This is a semi-generated file (see audio_syms.pl) */

/* $Id: aixppc_defines.c,v 1.4 2004/04/30 23:46:40 silvio Exp $ */


#define FL_STR_LEN 512

int
cat_if_possible( char *dest, char* src )
{
    if( (strlen(dest) + strlen(src) + 1) < FL_STR_LEN )
    {
        strcat( dest, src );
        return 0;
    }
    return 1;
}        

char*
status_flags( unsigned int value )
{
    char strg[FL_STR_LEN];
    sprintf( strg, "%u: ", value );
    if( value & AUDIO_FIXED )
    {
        if( cat_if_possible( strg, "AUDIO_FIXED | " ) )
            return strg;
    }
    if( value & AUDIO_LEFT_ALIGNED )
    {
        if( cat_if_possible( strg, "AUDIO_LEFT_ALIGNED | " ) )
            return strg;
    }
    if( value & AUDIO_RIGHT_ALIGNED )
    {
        if( cat_if_possible( strg, "AUDIO_RIGHT_ALIGNED | " ) )
            return strg;
    }
    if( value & AUDIO_TWOS_COMPLEMENT )
    {
        if( cat_if_possible( strg, "AUDIO_TWOS_COMPLEMENT | " ) )
            return strg;
    }
    if( value & AUDIO_SIGNED )
    {
        if( cat_if_possible( strg, "AUDIO_SIGNED | " ) )
            return strg;
    }
    if( value & AUDIO_BIG_ENDIAN )
    {
        if( cat_if_possible( strg, "AUDIO_BIG_ENDIAN | " ) )
            return strg;
    }
    if( value & AUDIO_PITCH )
    {
        if( cat_if_possible( strg, "AUDIO_PITCH | " ) )
            return strg;
    }
    if( value & AUDIO_INPUT )
    {
        if( cat_if_possible( strg, "AUDIO_INPUT | " ) )
            return strg;
    }
    if( value & AUDIO_OUTPUT )
    {
        if( cat_if_possible( strg, "AUDIO_OUTPUT | " ) )
            return strg;
    }
    if( value & AUDIO_MONITOR )
    {
        if( cat_if_possible( strg, "AUDIO_MONITOR | " ) )
            return strg;
    }
    if( value & AUDIO_VOLUME )
    {
        if( cat_if_possible( strg, "AUDIO_VOLUME | " ) )
            return strg;
    }
    if( value & AUDIO_VOLUME_DELAY )
    {
        if( cat_if_possible( strg, "AUDIO_VOLUME_DELAY | " ) )
            return strg;
    }
    if( value & AUDIO_BALANCE )
    {
        if( cat_if_possible( strg, "AUDIO_BALANCE | " ) )
            return strg;
    }
    if( value & AUDIO_BALANCE_DELAY )
    {
        if( cat_if_possible( strg, "AUDIO_BALANCE_DELAY | " ) )
            return strg;
    }
    if( value & AUDIO_TREBLE )
    {
        if( cat_if_possible( strg, "AUDIO_TREBLE | " ) )
            return strg;
    }
    if( value & AUDIO_BASS )
    {
        if( cat_if_possible( strg, "AUDIO_BASS | " ) )
            return strg;
    }
    if( value & AUDIO_BESTFIT_PROVIDED )
    {
        if( cat_if_possible( strg, "AUDIO_BESTFIT_PROVIDED | " ) )
            return strg;
    }
    if( value & AUDIO_LOAD_CODE )
    {
        if( cat_if_possible( strg, "AUDIO_LOAD_CODE | " ) )
            return strg;
    }
    strg[strlen(strg)-2] = 0;    return strg;
}

char*
operation_status_flags( unsigned int value )
{
    char strg[FL_STR_LEN];
    sprintf( strg, "%u: ", value );
    if( value & AUDIO_STOPPED )
    {
        if( cat_if_possible( strg, "AUDIO_STOPPED | " ) )
            return strg;
    }
    if( value & AUDIO_PLAYING )
    {
        if( cat_if_possible( strg, "AUDIO_PLAYING | " ) )
            return strg;
    }
    if( value & AUDIO_RECORDING )
    {
        if( cat_if_possible( strg, "AUDIO_RECORDING | " ) )
            return strg;
    }
    if( value & AUDIO_UNINITIALIZED )
    {
        if( cat_if_possible( strg, "AUDIO_UNINITIALIZED | " ) )
            return strg;
    }
    strg[strlen(strg)-2] = 0;    return strg;
}

char*
buffer_flags( unsigned int value )
{
    char strg[FL_STR_LEN];
    sprintf( strg, "%u: ", value );
    if( value & AUDIO_UNDERRUN )
    {
        if( cat_if_possible( strg, "AUDIO_UNDERRUN | " ) )
            return strg;
    }
    if( value & AUDIO_OVERRUN )
    {
        if( cat_if_possible( strg, "AUDIO_OVERRUN | " ) )
            return strg;
    }
    if( value & AUDIO_FIFO_UNDERRUN )
    {
        if( cat_if_possible( strg, "AUDIO_FIFO_UNDERRUN | " ) )
            return strg;
    }
    if( value & AUDIO_FIFO_OVERRUN )
    {
        if( cat_if_possible( strg, "AUDIO_FIFO_OVERRUN | " ) )
            return strg;
    }
    strg[strlen(strg)-2] = 0;    return strg;
}

char*
support_flags( unsigned int value )
{
    char strg[FL_STR_LEN];
    sprintf( strg, "%u: ", value );
    if( value & AUDIO_LEFT_CHANNEL )
    {
        if( cat_if_possible( strg, "AUDIO_LEFT_CHANNEL | " ) )
            return strg;
    }
    if( value & AUDIO_RIGHT_CHANNEL )
    {
        if( cat_if_possible( strg, "AUDIO_RIGHT_CHANNEL | " ) )
            return strg;
    }
    if( value & AUDIO_MUTE )
    {
        if( cat_if_possible( strg, "AUDIO_MUTE | " ) )
            return strg;
    }
    if( value & AUDIO_GAIN )
    {
        if( cat_if_possible( strg, "AUDIO_GAIN | " ) )
            return strg;
    }
    if( value & AUDIO_GAIN_DELAY )
    {
        if( cat_if_possible( strg, "AUDIO_GAIN_DELAY | " ) )
            return strg;
    }
    strg[strlen(strg)-2] = 0;    return strg;
}

char*
error_symbol( int value )
{
    char strg[FL_STR_LEN];
    switch( value )
    {
    case AUDIO_NO_PLAY: return "AUDIO_NO_PLAY";
    case AUDIO_NO_RECORD: return "AUDIO_NO_RECORD";
    case AUDIO_INVALID_REQUEST: return "AUDIO_INVALID_REQUEST";
    case AUDIO_CONFLICT: return "AUDIO_CONFLICT";
    case AUDIO_OVERLOADED: return "AUDIO_OVERLOADED";
    }
    sprintf(strg, "%d [unknown]", value);
    return strg;
}
        
    
char*
change_input_symbol( int value )
{
    char strg[FL_STR_LEN];
    switch( value )
    {
    case AUDIO_HIGH_GAIN_MIKE: return "AUDIO_HIGH_GAIN_MIKE";
    case AUDIO_LINE_1: return "AUDIO_LINE_1";
    case AUDIO_LINE_2: return "AUDIO_LINE_2";
    case AUDIO_LINES_1AND2: return "AUDIO_LINES_1AND2";
    case AUDIO_LOW_GAIN_MIKE: return "AUDIO_LOW_GAIN_MIKE";
    case AUDIO_AUX1: return "AUDIO_AUX1";
    case AUDIO_AUX2: return "AUDIO_AUX2";
    case AUDIO_AUX1_LEFT: return "AUDIO_AUX1_LEFT";
    case AUDIO_AUX1_RIGHT: return "AUDIO_AUX1_RIGHT";
    case AUDIO_AUX2_LEFT: return "AUDIO_AUX2_LEFT";
    case AUDIO_AUX2_RIGHT: return "AUDIO_AUX2_RIGHT";
    case AUDIO_RECORD_MONITOR: return "AUDIO_RECORD_MONITOR";
    case AUDIO_ALL_LINES: return "AUDIO_ALL_LINES";
    }
    sprintf(strg, "%d [unknown]", value);
    return strg;
}

char*
change_output_symbol( int value )
{
    char strg[FL_STR_LEN];
    switch( value )
    {
    case AUDIO_EXTERNAL_SPEAKER: return "AUDIO_EXTERNAL_SPEAKER";
    case AUDIO_INTERNAL_SPEAKER: return "AUDIO_INTERNAL_SPEAKER";
    case AUDIO_OUTPUT_1: return "AUDIO_OUTPUT_1";
    case AUDIO_INTERNAL_SPEAKER_ON: return "AUDIO_INTERNAL_SPEAKER_ON";
    case AUDIO_INTERNAL_SPEAKER_OFF: return "AUDIO_INTERNAL_SPEAKER_OFF";
    case AUDIO_INTERNAL_AUDIO_ON: return "AUDIO_INTERNAL_AUDIO_ON";
    case AUDIO_INTERNAL_AUDIO_OFF: return "AUDIO_INTERNAL_AUDIO_OFF";
    }
    sprintf(strg, "%d [unknown]", value);
    return strg;
}

char*
change_monitor_symbol( int value )
{
    char strg[FL_STR_LEN];
    switch( value )
    {
    case AUDIO_MONITOR_OFF: return "AUDIO_MONITOR_OFF";
    case AUDIO_MONITOR_UNCOMPRESSED: return "AUDIO_MONITOR_UNCOMPRESSED";
    case AUDIO_MONITOR_COMPRESSED: return "AUDIO_MONITOR_COMPRESSED";
    }
    sprintf(strg, "%d [unknown]", value);
    return strg;
}

char*
mode_symbol( int value )
{
    char strg[FL_STR_LEN];
    switch( value )
    {
    case AUDIO_ADPCM: return "AUDIO_ADPCM";
    case AUDIO_PCM: return "AUDIO_PCM";
    case AUDIO_MU_LAW: return "AUDIO_MU_LAW";
    case AUDIO_A_LAW: return "AUDIO_A_LAW";
    case AUDIO_SOURCE_MIX: return "AUDIO_SOURCE_MIX";
    case AUDIO_IMA_ADPCM: return "AUDIO_IMA_ADPCM";
    case AUDIO_ANALOG_MIX: return "AUDIO_ANALOG_MIX";
    }
    sprintf(strg, "%d [unknown]", value);
    return strg;
}

char*
get_channels_symbol( int value )
{
    char strg[FL_STR_LEN];
    switch( value )
    {
    case AUDIO_HIGH_GAIN_MIC: return "AUDIO_HIGH_GAIN_MIC";
    case AUDIO_LOW_GAIN_MIC: return "AUDIO_LOW_GAIN_MIC";
    case AUDIO_LINE1_IN: return "AUDIO_LINE1_IN";
    case AUDIO_MONO_IN: return "AUDIO_MONO_IN";
    case AUDIO_AUX1_IN: return "AUDIO_AUX1_IN";
    case AUDIO_AUX2_IN: return "AUDIO_AUX2_IN";
    case AUDIO_HOST_INPUT: return "AUDIO_HOST_INPUT";
    case AUDIO_LINE1_OUT: return "AUDIO_LINE1_OUT";
    case AUDIO_MONO_OUT: return "AUDIO_MONO_OUT";
    case AUDIO_HOST_OUTPUT: return "AUDIO_HOST_OUTPUT";
    }
    sprintf(strg, "%d [unknown]", value);
    return strg;
}

void
print_status( audio_status *as )
{
    masc_log_message( MAS_VERBLVL_DEBUG, "--[ RESULTS OF AUDIO_STATUS ]-------" );
    masc_log_message( MAS_VERBLVL_DEBUG, "sample rate           %d", as->srate );
    masc_log_message( MAS_VERBLVL_DEBUG, "bits / sample         %d", as->bits_per_sample );
    masc_log_message( MAS_VERBLVL_DEBUG, "block size            %d", as->bsize );
    masc_log_message( MAS_VERBLVL_DEBUG, "mode                  %s", mode_symbol(as->mode) );
    masc_log_message( MAS_VERBLVL_DEBUG, "channels              %d", as->channels );
    masc_log_message( MAS_VERBLVL_DEBUG, "flags                 %s", status_flags(as->flags) );
    masc_log_message( MAS_VERBLVL_DEBUG, "operation             %s", operation_status_flags(as->operation) );
    masc_log_message( MAS_VERBLVL_DEBUG, "audio_change"      );
    masc_log_message( MAS_VERBLVL_DEBUG, "  input               %s", change_input_symbol( as->change.input) );
    masc_log_message( MAS_VERBLVL_DEBUG, "  output              %s", change_output_symbol( as->change.output) );
    masc_log_message( MAS_VERBLVL_DEBUG, "  monitor             %s", change_monitor_symbol( as->change.monitor) );
    masc_log_message( MAS_VERBLVL_DEBUG, "  volume              %d", as->change.volume );
    masc_log_message( MAS_VERBLVL_DEBUG, "  volume_delay        %d", as->change.volume_delay );
    masc_log_message( MAS_VERBLVL_DEBUG, "  balance             %d", as->change.balance );
    masc_log_message( MAS_VERBLVL_DEBUG, "  balance_delay       %d", as->change.balance_delay);
    masc_log_message( MAS_VERBLVL_DEBUG, "  treble              %d", as->change.treble );
    masc_log_message( MAS_VERBLVL_DEBUG, "  bass                %d", as->change.bass );
    masc_log_message( MAS_VERBLVL_DEBUG, "  pitch               %d", as->change.pitch );
    masc_log_message( MAS_VERBLVL_DEBUG, "------------------------------------" );
}

void 
print_init( audio_init *ai )
{
    masc_log_message( MAS_VERBLVL_DEBUG, "--[ RESULTS OF AUDIO_INIT ]---------" );
    masc_log_message( MAS_VERBLVL_DEBUG, "srate                 %d", ai->srate               );
    masc_log_message( MAS_VERBLVL_DEBUG, "bits_per_sample       %d", ai->bits_per_sample     );
    masc_log_message( MAS_VERBLVL_DEBUG, "bsize                 %d", ai->bsize               );
    masc_log_message( MAS_VERBLVL_DEBUG, "mode                  %s", mode_symbol(ai->mode)   );
    masc_log_message( MAS_VERBLVL_DEBUG, "channels              %d", ai->channels            );
    masc_log_message( MAS_VERBLVL_DEBUG, "position_resolution   %d", ai->position_resolution );
    if( ai->loadpath )
        masc_log_message( MAS_VERBLVL_DEBUG, "loadpath              %s", ai->loadpath );

    masc_log_message( MAS_VERBLVL_DEBUG, "flags                 %s", status_flags(ai->flags) );
    masc_log_message( MAS_VERBLVL_DEBUG, "operation             %s",
            (ai->operation==AUDIO_PLAY) ? "AUDIO_PLAY" :  (ai->operation==AUDIO_RECORD) ? "AUDIO_RECORD" : "[unknown]" );


    masc_log_message( MAS_VERBLVL_DEBUG, "rc                    %s", (ai->rc==0) ? "0 [ok]" : error_symbol(ai->rc) );              
    masc_log_message( MAS_VERBLVL_DEBUG, "slot_number           %d", ai->slot_number ); 
    masc_log_message( MAS_VERBLVL_DEBUG, "device_id             %d", ai->device_id ); 
    masc_log_message( MAS_VERBLVL_DEBUG, "reserved              %u", ai->reserved );
    masc_log_message( MAS_VERBLVL_DEBUG, "------------------------------------" );
}

void
print_buffer( audio_buffer *ab )
{
    masc_log_message( MAS_VERBLVL_DEBUG, "--[ RESULTS OF BUFFER STATUS ]------" );
    masc_log_message( MAS_VERBLVL_DEBUG, "flags                   %s", (ab->flags)?buffer_flags(ab->flags):"0 [ok]" );
    masc_log_message( MAS_VERBLVL_DEBUG, "read_buf_size           %u", ab->read_buf_size );
    masc_log_message( MAS_VERBLVL_DEBUG, "write_buf_size          %u", ab->write_buf_size );
    masc_log_message( MAS_VERBLVL_DEBUG, "read_buf_time           %u", ab->read_buf_time );
    masc_log_message( MAS_VERBLVL_DEBUG, "write_buf_time          %u", ab->write_buf_time );
    masc_log_message( MAS_VERBLVL_DEBUG, "read_buf_max            %u", ab->read_buf_max );
    masc_log_message( MAS_VERBLVL_DEBUG, "write_buf_max           %u", ab->write_buf_max );
    masc_log_message( MAS_VERBLVL_DEBUG, "position                %u", ab->position );
    masc_log_message( MAS_VERBLVL_DEBUG, "position_type           %s", (ab->position_type==AUDIO_POS_MSECS)?"AUDIO_POS_MSECS":"[unknown]" );
    masc_log_message( MAS_VERBLVL_DEBUG, "read_buf_cap            %d", ab->read_buf_cap );
    masc_log_message( MAS_VERBLVL_DEBUG, "write_buf_cap           %d", ab->write_buf_cap );
    masc_log_message( MAS_VERBLVL_DEBUG, "request_buf_cap         %d", ab->request_buf_cap );
    masc_log_message( MAS_VERBLVL_DEBUG, "------------------------------------" );
}


/* FIXME: this routine doesn't check the ends of its string buffer strg... */
void
print_get_channels( audio_get_channels *agc )
{
    int i;
    char strg[2*FL_STR_LEN];
    char strg2[FL_STR_LEN];

    masc_log_message( MAS_VERBLVL_DEBUG, "--[ RESULTS OF AUDIO_GET_CHANNELS ]-" );
    masc_log_message( MAS_VERBLVL_DEBUG, "device_owner          %u", agc->device_owner  );
    masc_log_message( MAS_VERBLVL_DEBUG, "driver_version        %u", agc->driver_version);
    masc_log_message( MAS_VERBLVL_DEBUG, "channel_count         %u", agc->channel_count );
    masc_log_message( MAS_VERBLVL_DEBUG, "rate_count            %u", agc->rate_count    );
    masc_log_message( MAS_VERBLVL_DEBUG, "format_count          %u", agc->format_count  );
    masc_log_message( MAS_VERBLVL_DEBUG, "reserved              %u", agc->reserved      );

    sprintf( strg, "formats               " );
    for( i=0; i< agc->format_count ; i++ )
    {
        if( i>0 ) strcat( strg, "                      " );
        strcat( strg, mode_symbol(agc->formats[i]) );
        masc_log_message( MAS_VERBLVL_DEBUG, "%s", strg );
        *strg = 0;
    }
                
                
    sprintf( strg, "rates                 " );
    for( i=0; i<agc->rate_count; i++ )
    {
        sprintf( strg2, "%d ", agc->rates[i] );
        strcat( strg, strg2 );
    }
    masc_log_message( MAS_VERBLVL_DEBUG, "%s", strg );
                
                
    sprintf( strg, "channel_type          " );
    for( i=0; i< agc->channel_count ; i++ )
    {
        if( i>0 ) strcat( strg, "                      " );
        strcat( strg, get_channels_symbol(agc->channel_type[i]) );
        masc_log_message( MAS_VERBLVL_DEBUG, "%s", strg );
        *strg = 0;
    }
                
    sprintf( strg, "support               " );
    for( i=0; i< agc->channel_count ; i++ )
    {
        if( i>0 ) strcat( strg, "                      " );
        strcat( strg, support_flags(agc->support[i]) );
        masc_log_message( MAS_VERBLVL_DEBUG, "%s", strg );
        *strg = 0;
    }
                
    masc_log_message( MAS_VERBLVL_DEBUG, "return_code           %d", agc->return_code   );
    masc_log_message( MAS_VERBLVL_DEBUG, "device_id             %s", (agc->device_id==AUDIO_BASE_4231 ? "AUDIO_BASE_4231" : (agc->device_id==AUDIO_ADAPTER_4231) ? "AUDIO_ADAPTER_4231" : "[??!]") );
                
    
    masc_log_message( MAS_VERBLVL_DEBUG, "------------------------------------" );
}

